import datetime
import json
import logging
import os

from sendgrid import SendGridAPIClient

from backup_sendgrid_templates.strategies import FileSystemStrategy


class BackupSendgridTemplatesService:
    def __init__(self, backup_only_active_versions=False, api_key=None, strategy=None):
        self.backup_only_active_versions = backup_only_active_versions
        _api_key = api_key or os.environ.get("SENDGRID_API_KEY")
        if not _api_key:
            raise Exception("SENDGRID_API_KEY environment variable not set")
        self.sendgrid_client = SendGridAPIClient(_api_key).client
        self.now = datetime.datetime.utcnow().isoformat()
        self.strategy = strategy or self.get_default_strategy()

    @staticmethod
    def get_default_strategy():
        return FileSystemStrategy()

    def run(self):
        """Backup sendgrid templates"""
        logging.info("Backupping sendgrid templates...")
        templates_metadata = self.get_templates_metadata()
        template_count, version_count, error_count = 0, 0, 0
        for template_data in templates_metadata:
            templates, versions, errors = self.retrieve_template_and_backup_it(
                template_data
            )
            template_count += templates
            version_count += versions
            error_count += errors
        logging.info("\nSendgrid templates backupped.")
        logging.info(
            f"{template_count} templates, {version_count} versions, {error_count} errors."
        )
        return {
            "template_count": template_count,
            "version_count": version_count,
            "error_count": error_count,
        }

    def get_templates_metadata(self):
        """Get templates from sendgrid"""
        # As of today, Sendgrid docs wrongly report the parameter of get() method if you want to retrieve all the
        # existing templates.
        response = self.sendgrid_client.templates.get(
            None, {"generations": "legacy,dynamic"}
        )
        if response.status_code == 200:
            response_body = json.loads(response.body)
            templates = response_body["templates"]
            logging.info(f"Found {len(templates)} templates")
            logging.debug(templates)
            return templates
        logging.info(
            f"Request to sendgrid failed: {response.body} (status code: {response.status_code})"
        )
        return []

    def retrieve_template_and_backup_it(self, template):
        """Backup the provided template depending on the selected strategy"""
        uploaded_versions = 0
        errors = 0

        template_id = template["id"]
        template_name = template["name"]
        logging.info(f"TEMPLATE {template_id} - {template_name}")

        response = self.sendgrid_client.templates._(template_id).get()
        if response.status_code != 200:
            logging.info(
                f"Error retrieving template {template_id}, nothing uploaded on S3..."
            )
            return 0, 0, 1

        # template_data contains the html content for each version.
        template_data = json.loads(response.body)

        for version_data in template_data["versions"]:
            if self.backup_only_active_versions and not version_data["active"]:
                logging.info(
                    f"Skipping inactive version \"{version_data['name']}\" (id {version_data['id']})"
                )
                continue

            if self.strategy.backup_template_version(
                template_name, version_data, self.now
            ):
                uploaded_versions += 1

        # if at least 1 version has been uploaded on S3 then increment also the number of uploaded templates.
        uploaded_templates = 1 if uploaded_versions > 0 else 0
        logging.info(
            f"{uploaded_templates} templates uploaded, {uploaded_versions} versions uploaded, {errors} errors"
        )
        return uploaded_templates, uploaded_versions, errors



import logging

from backup_sendgrid_templates.service import BackupSendgridTemplatesService
from backup_sendgrid_templates.strategies import FileSystemStrategy, S3Strategy

logging.basicConfig(level=logging.INFO)


def backup_templates_on_file_system(api_key: str = None):
    """
    Backup all existing templates found in a Sendgrid account. Save the templates and their versions on a local file
    system.
    Args:
        api_key (str): sendgrid api key. If not provided, the environment variable SENDGRID_API_KEY is used.
    Returns:
        None
    """
    BackupSendgridTemplatesService(api_key=api_key, strategy=FileSystemStrategy()).run()


def backup_templates_on_s3(api_key: str = None):
    """
    Backup all existing templates found in a Sendgrid account. Save the templates and their versions on a S3 bucket.
    Args:
        api_key (str): sendgrid api key. If not provided, the environment variable SENDGRID_API_KEY is used.
    Returns:
        None
    """
    BackupSendgridTemplatesService(api_key=api_key, strategy=S3Strategy()).run()


if __name__ == "__main__":
    backup_templates_on_file_system()
    # backup_templates_on_s3()



import json
import logging
import os
from abc import ABC, abstractmethod
from pathlib import Path

from backup_sendgrid_templates.aws_utils import create_file_on_s3


class Strategy(ABC):
    """
    The Strategy interface declares operations common to all supported versions
    of some algorithm.

    The Context uses this interface to call the algorithm defined by Concrete
    Strategies.
    See https://refactoring.guru/design-patterns/strategy/python/example
    """

    @abstractmethod
    def backup_template_version(
        self, template_name: str, version_data: any, backup_root: str
    ) -> bool:
        pass


class FileSystemStrategy(Strategy):
    def __init__(self, base_path: str = None):
        self._base_path = base_path or os.environ.get("SENDGRID_TEMPLATES_BASE_PATH")
        if not self._base_path:
            raise Exception("SENDGRID_TEMPLATES_BASE_PATH environment variable not set")

    @staticmethod
    def create_file_on_file_system(
        version_path: Path, file_name: str, file_content: str
    ) -> bool:
        (version_path / file_name).write_text(file_content, encoding="utf-8")
        return True

    def backup_template_version(
        self, template_name: str, version_data: any, backup_root: str
    ) -> bool:
        template_id = version_data[
            "template_id"
        ]  # template_id is repeated inside the version object
        version_name = version_data["name"]
        active = version_data["active"]
        html_content = version_data.pop("html_content")

        version_path = Path(self._base_path) / backup_root
        version_path.mkdir(parents=True, exist_ok=True)
        logging.info(f"Storing template files in directory {version_path}")

        file_prefix = (
            f"{template_id}_{template_name}{'_active' if active else ''}_{version_name}"
        )

        meta_file_name = f"{file_prefix}_metadata.json"
        _meta_created = self.create_file_on_file_system(
            version_path, meta_file_name, json.dumps(version_data)
        )
        content_file_name = f"{file_prefix}_content.html"
        _content_created = self.create_file_on_file_system(
            version_path, content_file_name, html_content
        )
        return _meta_created and _content_created


class S3Strategy(Strategy):
    def __init__(self, bucket_name: str = None):
        self._bucket_name = bucket_name or os.environ.get(
            "SENDGRID_TEMPLATES_BUCKET_NAME"
        )
        if not self._bucket_name:
            raise Exception(
                "SENDGRID_TEMPLATES_BUCKET_NAME environment variable not set"
            )

    def backup_template_version(
        self, template_name: str, version_data: any, backup_root: str
    ) -> bool:
        template_id = version_data[
            "template_id"
        ]  # template_id is repeated inside the version object
        version_name = version_data["name"]
        active = version_data["active"]
        html_content = version_data.pop("html_content")

        key_prefix = f"{backup_root}/{template_id}_{template_name}{'_active' if active else ''}_{version_name}"

        meta_key = f"{key_prefix}_metadata.json"
        _meta_created = create_file_on_s3(
            self._bucket_name, meta_key, json.dumps(version_data)
        )

        html_content_key = f"{key_prefix}_content.html"
        _content_created = create_file_on_s3(
            self._bucket_name, html_content_key, html_content
        )
        return _meta_created and _content_created



import logging

import boto3


def create_file_on_s3(bucket_name, object_key, file_content):
    session = boto3.Session()
    # Creating S3 Resource From the Session.
    s3 = session.resource("s3")
    s3_object = s3.Object(bucket_name, object_key)
    result = s3_object.put(Body=bytearray(file_content, encoding="utf-8"))
    res = result.get("ResponseMetadata")
    if res.get("HTTPStatusCode") == 200:
        logging.info(f"File {object_key} uploaded successfully")
        return True
    logging.info(f"File {object_key} not uploaded.")
    return False
